(* Sudoku solver

    J. Rathlev, D-24222, Schwentinental, (info(a)rathlev-home.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   Vers. 1 - Mar. 2006
   Vers. 2 - Jan. 2008 - step mode, multilanguage
   Vers. 3 - Dec. 2013 - print function

   Command line option:
     /lang:xx         (xx = de, en)
  *)

unit SuMain;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes,
  Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.ExtCtrls, Vcl.Grids,
  Vcl.StdCtrls, Vcl.Buttons, Vcl.ComCtrls, Vcl.Menus;

const
  CopRgt = ' 2008-2016 - J. Rathlev, D-24222 Schwentinental';
  EmailAdr = 'kontakt(a)rathlev-home.de';

  StackMax = 81;

type
  TDigit = 0..9;
  TSuValues = set of TDigit;

  TData = record
    Count : integer;
    Values : TSuValues;
    end;

  TDigitGrid = array [1..9,1..9] of TDigit;
  TDataGrid = array [1..9,1..9] of TData;

  TGridStackEntry = record
    r,c    : integer;
    CField : TData;
    DGrid  : TDigitGrid;
    end;

  TGridStack = array [1..StackMax] of TGridStackEntry;

  TCheckGrid = (cgCont,cgFound,cgError);

const
  SudExt = 'sud';
  TestGrid : TDigitGrid = (
      (0,2,6,4,5,8,3,0,0),(1,7,0,0,0,0,0,4,0),(0,8,0,0,0,0,0,0,0),
      (0,0,0,0,0,0,9,8,0),(0,0,0,5,9,0,1,0,4),(7,0,0,2,0,1,0,5,0),
      (0,0,0,0,4,0,0,3,0),(0,0,0,8,0,0,5,0,0),(6,0,0,0,0,7,0,9,1));

type
  TSuForm = class(TForm)
    pnlFeld: TPanel;
    btnStart: TBitBtn;
    Feld: TStringGrid;
    StatusBar: TStatusBar;
    btnLoad: TBitBtn;
    btnSave: TBitBtn;
    btnClear: TBitBtn;
    btnStop: TBitBtn;
    OpenDialog: TOpenDialog;
    SaveDialog: TSaveDialog;
    btnReset: TBitBtn;
    lblFile: TLabel;
    cbStep: TCheckBox;
    MainMenu: TMainMenu;
    itmPuzzle: TMenuItem;
    itmLoad: TMenuItem;
    itmSave: TMenuItem;
    itmNew: TMenuItem;
    itmReset: TMenuItem;
    N1: TMenuItem;
    itmQuit: TMenuItem;
    itmInfo: TMenuItem;
    itmSolve: TMenuItem;
    N2: TMenuItem;
    irmPrint: TMenuItem;
    btnPrint: TBitBtn;
    itmPrefs: TMenuItem;
    itmLanguage: TMenuItem;
    itmGerman: TMenuItem;
    itmEnglish: TMenuItem;
    itmPrinterSet: TMenuItem;
    btnInfo: TBitBtn;
    btnQuit: TBitBtn;
    PrinterSetupDialog: TPrinterSetupDialog;
    paButtons: TPanel;
    N3: TMenuItem;
    itmTitel: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure btnStartClick(Sender: TObject);
    procedure FeldGetEditMask(Sender: TObject; ACol, ARow: Integer;
      var Value: String);
    procedure btnClearClick(Sender: TObject);
    procedure btnStopClick(Sender: TObject);
    procedure btnLoadClick(Sender: TObject);
    procedure btnSaveClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FeldDrawCell(Sender: TObject; ACol, ARow: Integer;
      Rect: TRect; State: TGridDrawState);
    procedure btnResetClick(Sender: TObject);
    procedure FeldKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FeldKeyPress(Sender: TObject; var Key: Char);
    procedure itmInfoClick(Sender: TObject);
    procedure itmQuitClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure itmEnuClick(Sender: TObject);
    procedure itmGerClick(Sender: TObject);
    procedure btnPrintClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure itmPrinterSetClick(Sender: TObject);
    procedure itmTitelClick(Sender: TObject);
    procedure FeldEnter(Sender: TObject);
    procedure FeldExit(Sender: TObject);
  private
    { Private-Deklarationen }
    ProgVersName,
    ProgVersDate,
    AppPath,UserPath,
    IniName,LastPath,
    TextName,Comment,
    PrtName          : string;
    DataLoaded,
    Stop,Step,Run    : boolean;
    TCount,
    BCount,
    CCount     : cardinal;
    procedure SetEditMode (Status : boolean);
    procedure LoadData (FName : string);
    procedure ShowStatus;
    procedure OpenPrint (const FName : string);
    procedure PrintFile (n : integer; const FName : string);
    procedure ClosePrint;
  public
    { Public-Deklarationen }
  end;

var
  SuForm: TSuForm;

implementation

{$R *.dfm}

uses System.IniFiles, Winapi.ShlObj, Vcl.Printers, WinUtils, StringUtils,
  WinShell, GnuGetText, InitProg, LangUtils, FileUtils;

{-----------------------------------------------------------------------}
type
  TPageLayout = record
     Left,Top,Size,Space,HDist,VDist : integer;
    end;

const
  // Standardlayout (Angaben in mm)
  FieldSz = 8;
  DefPage : TPageLayout = (Left : 15; Top : 15;
                           Size : 9*FieldSz; Space : 10;
                           HDist : 26; VDist : 10);
  PtToMm = 0.353;   // 1 Pt in mm

  (* INI-Sektionen *)
  CfgSekt = 'Config';

  (* INI-Variablen *)
  iniLeft = 'Left';
  iniTop = 'Top';
  iniLast = 'Path';
  iniComment = 'Comment';
  iniPrtName = 'Printer';

procedure TSuForm.FormCreate(Sender: TObject);
var
  IniFile : TIniFile;
begin
  TranslateComponent(self);
  InitPaths(AppPath,UserPath);
  InitVersion(Application.Title,'Vers. 3.0',CopRgt,2,2,ProgVersName,ProgVersDate);
  IniName:=Erweiter(AppPath,PrgName,'ini');
  IniFile:=TIniFile.Create(IniName);
  with IniFile do begin
    Left:=ReadInteger(CfgSekt,iniLeft,Left);
    Top:=ReadInteger(CfgSekt,iniTop,Top);
    LastPath:=ReadString(CfgSekt,iniLast,UserPath);
    PrtName:=ReadString (CfgSekt,iniPrtName,'');
    Free;
    end;
  ActiveControl:=btnStart;
  SetEditMode(true);
  lblFile.Caption:=_('New puzzle');
  Step:=false; Run:=false; DataLoaded:=false;
  btnReset.Enabled:=DataLoaded;
  end;

procedure TSuForm.FormDestroy(Sender: TObject);
var
  IniFile : TIniFile;
begin
  IniFile:=TIniFile.Create(IniName);
  with IniFile do begin
    WriteInteger(CfgSekt,iniLeft,Left);
    WriteInteger(CfgSekt,iniTop,Top);
    WriteString(CfgSekt,iniLast,LastPath);
    WriteString (CfgSekt,iniPrtName,PrtName);
    Free;
    end;
  end;

procedure TSuForm.FormShow(Sender: TObject);
begin
  with Printer do if Printers.Count>0 then begin
    PrinterIndex:=Printers.IndexOf(PrtName);
    PrtName:=Printers[PrinterIndex];
    Orientation:=poPortrait;
    btnPrint.Enabled:=true;
    end
  else begin
    btnPrint.Enabled:=false;
    PrtName:='';
    end;
  Feld.SetFocus;
  end;

{ ------------------------------------------------------------------- }
procedure TSuForm.itmQuitClick(Sender: TObject);
begin
  Stop:=true; Step:=false;
  Close;
  end;

procedure TSuForm.itmTitelClick(Sender: TObject);
begin
  if InputQuery(_('Sudoku puzzle'),_('Comment:'),Comment) then
    lblFile.Hint:=Comment;
  end;

procedure TSuForm.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  CanClose:=not Run;
  end;

procedure TSuForm.itmInfoClick(Sender: TObject);
begin
  InfoDialog(BottomRightPos(btnInfo),ProgVersName+' - '+ProgVersDate+sLineBreak+CopRgt
           +sLineBreak+'E-Mail: '+EmailAdr);
  end;

{-----------------------------------------------------------------------}
procedure TSuForm.FeldDrawCell(Sender: TObject; ACol, ARow: Integer;
  Rect: TRect; State: TGridDrawState);
begin
  with (Sender as TStringGrid),Canvas do begin
    if EditorMode then Brush.Color:=clWhite
    else if Run then begin
      Brush.Color:=TColor($00C8C8FF); // lightred
      if Step then begin
        if (integer(Objects[ACol,ARow]) and 4<>0) then Brush.Color:=TColor($00B9FFB9) //lightgreen
        else if (integer(Objects[ACol,ARow]) and 2<>0) then Brush.Color:=TColor($0095FFFF);     // yellow
        end;
      end
    else Brush.Color:=TColor($0095FFFF);
    Pen.Color:=TColor($00B37800);
    if integer(Objects[ACol,ARow]) and 1<>0 then begin
      Font.Color:=clRed;
      end
    else begin
      Font.Color:=clBlack;
      end;
    with Font do begin
      Size:=10; Style:=[fsBold];
      end;
    FillRect (Rect);
    with Rect do begin
      with Pen do if (ACol mod 3=2) then Width:=2 else Width:=1;
      if ACol<8 then begin
        MoveTo(Right,Top); LineTo(Right,Bottom);
        end;
      with Pen do if (ARow mod 3=2) then Width:=2 else Width:=1;
      if ARow<8 then begin
        MoveTo(Right,Bottom); LineTo(Left,Bottom);
        end;
      end;
    with Rect do TextOut(Left+8,Top+4,Cells[ACol,Arow]);
    end;
  end;

procedure TSuForm.FeldEnter(Sender: TObject);
begin
  with Feld do Options:=Options+[goEditing];
  end;

procedure TSuForm.FeldExit(Sender: TObject);
begin
  with Feld do Options:=Options-[goEditing];
  end;

procedure TSuForm.FeldGetEditMask(Sender: TObject; ACol, ARow: Integer;
  var Value: String);
begin
  Value:='9';
  end;

{-----------------------------------------------------------------------}
procedure TSuForm.SetEditMode (Status : boolean);
var
  i : integer;
begin
  with Feld do begin
//    if Status then Options:=Options+[goAlwaysShowEditor]
//    else Options:=Options-[goAlwaysShowEditor];
    EditorMode:=Status;
    end;
  with paButtons do for i:=0 to ControlCount-1 do with Controls[i] do
    if Tag=0 then Enabled:=Status;
  cbStep.Enabled:=Status;
  btnStop.Enabled:=not Status;
  end;

procedure TSuForm.LoadData (FName : string);
var
  ft  : TextFile;
  i,j : integer;
  h,s,t : string;
  err : boolean;
begin
  AssignFile(ft,FName); reset(ft); err:=false;
  i:=0; h:='';
  while not Eof(ft) do begin
    readln(ft,s);
    s:=TrimLeft(s);
    if (length(s)>0) and (s[1]<>';') and (i<9) then with Feld do begin
      Rows[i].Clear;
      for j:=0 to 8 do begin
        Objects[j,i]:=pointer(0);
        t:=Trim(ReadNxtStr(s,','));
        if length(t)=0 then Cells[j,i]:=''
        else begin
          try StrToInt(t) except err:=true; end;
          Cells[j,i]:=t;
          end;
        end;
      inc(i);
      end
    else h:=Trim(Copy(s,2,length(s)-1));
    end;
  CloseFile(ft);
  if err then ErrorDialog(Caption,_('Error reading data from:')+sLineBreak+FName)
  else begin
    TextName:=FName;
    lblFile.Caption:=StripPath(TextName,55);
    Comment:=h;
    lblFile.Hint:=h;
    DataLoaded:=true;
    SetEditMode(true);
    StatusBar.Panels[0].Text:='';
    end;
  Step:=false;
  end;

{-----------------------------------------------------------------------}
procedure TSuForm.btnClearClick(Sender: TObject);
var
  i,j : integer;
begin
  DataLoaded:=false;
  lblFile.Caption:=_('New puzzle');
  with Feld do for i:=0 to 8 do for j:=0 to 8 do begin
    Cells[i,j]:='';
    Objects[i,j]:=pointer(0);
    end;
  SetEditMode(true);
  btnReset.Enabled:=false;
  StatusBar.Panels[0].Text:='';
  end;

procedure TSuForm.btnResetClick(Sender: TObject);
begin
  if FileExists(TextName) then LoadData (TextName);
  end;

procedure TSuForm.btnStopClick(Sender: TObject);
begin
  Stop:=true; Step:=false;
  end;

procedure TSuForm.btnLoadClick(Sender: TObject);
begin
  with OpenDialog do begin
    if length(TextName)>0 then InitialDir:=ExtractFilePath(TextName)
    else InitialDir:=LastPath;
    Filename:='';
    Options:=Options-[ofAllowMultiSelect];
    Title:=_('Load Sudoku puzzle');
    if Execute then begin
      LastPath:=ExtractFilePath(Filename);
      Stop:=true; Step:=false;
      LoadData (Filename);
      btnReset.Enabled:=true;
      end;
    end;
  end;

procedure TSuForm.itmPrinterSetClick(Sender: TObject);
begin
  if PrinterSetupDialog.Execute then with printer do PrtName:=Printers[PrinterIndex];
  end;


procedure TSuForm.OpenPrint (const FName : string);
begin
  with Printer do begin
    Title:=DelExt(ExtractFilename(FName));
    Canvas.Font.Name:='Arial';
    BeginDoc;
    end;
  end;

procedure TSuForm.PrintFile (n : integer; const FName : string);
var
  i,j,k,
  x0,y0,x1,y1 : integer;
  ft          : TextFile;
  h,s,t       : string;
  f           : array[0..8,0..8] of byte;
begin
  AssignFile(ft,FName); reset(ft);
  i:=0;
  while not Eof(ft) do begin
    readln(ft,s);
    s:=TrimLeft(s);
    if (length(s)>0) and (s[1]<>';') and (i<9) then begin
      for j:=0 to 8 do begin
        t:=Trim(ReadNxtStr(s,','));
        if (length(t)=0) or not TryStrToInt(t,k) then f[j,i]:=0
        else f[j,i]:=k;
        end;
      inc(i);
      end
    else h:=Trim(Copy(s,2,length(s)-1));
    end;
  CloseFile(ft);
  with Printer do begin
    with Canvas do begin
      Pen.Color:=clBlack;
      TextOut (0,0,'');
      SetMapMode (Handle,MM_LOMETRIC);
      if (n>0) and ((n mod 6)=0) then NewPage;
      n:=n mod 6;
      with DefPage do begin
        x0:=(Left+(n mod 2)*(Size+HDist))*10;
        y0:=-(Top+(n div 2)*(Size+Space+VDist))*10;
        x1:=x0+Size*10; y1:=y0-Size*10;
        end;
      Pen.Width:=7;
      RecTangle(x0,y0,x1,y1);
      for i:=1 to 8 do begin
        if i mod 3 = 0 then Pen.Width:=5 else Pen.Width:=3;
        MoveTo(x0+10*i*FieldSz,y0); LineTo(x0+10*i*FieldSz,y1);
        MoveTo(x0,y0-10*i*FieldSz); LineTo(x1,y0-10*i*FieldSz);
        end;
      Font.Height:=round(8*FieldSz);
      for i:=0 to 8 do begin
        if i mod 3 = 0 then Pen.Width:=5 else Pen.Width:=3;
        y1:=y0-10*(i*FieldSz+round(0.1*FieldSz));
        for j:=0 to 8 do if (f[j,i]>0) then begin
          s:=IntToStr(f[j,i]);
          TextOut(x0+10*(j*FieldSz+(FieldSz div 2))-TextWidth(s) div 2,y1,s);
          end;
        end;
      Font.Height:=round(5*FieldSz);
      with DefPage do
        TextOut(x0+10*(Size div 2)-TextWidth(h) div 2,y0-10*(Size+round(0.3*Space)),h);
      end;
    end;
  end;

procedure TSuForm.ClosePrint;
begin
  Printer.EndDoc;
  end;

procedure TSuForm.btnPrintClick(Sender: TObject);
var
  i : integer;
begin
  with OpenDialog do begin
    if length(TextName)>0 then InitialDir:=ExtractFilePath(TextName)
    else InitialDir:=LastPath;
    Filename:='';
    Options:=Options+[ofAllowMultiSelect];
    Title:=_('Print Sudoku puzzles');
    if Execute then if Files.Count>0 then begin
      OpenPrint (Files[0]);
      with Files do for i:=0 to Count-1 do PrintFile(i,Strings[i]);
      ClosePrint;
      end;
    end;
  end;

procedure TSuForm.btnSaveClick(Sender: TObject);
var
  ft  : TextFile;
  i,j : integer;
  s   : string;
begin
  with SaveDialog do begin
    if length(TextName)>0 then InitialDir:=ExtractFilePath(TextName)
    else InitialDir:=LastPath;
    Filename:=ExtractFileName(TextName);
    Title:=_('Save Sudoku puzzle');
    if Execute then begin
      LastPath:=ExtractFilePath(Filename);
      TextName:=Filename;
      if (length(Comment)=0) and InputQuery(_('Sudoku puzzle'),_('Comment:'),Comment) then s:=Comment
      else s:='';
      AssignFile(ft,TextName); rewrite(ft);
      if length(s)>0 then writeln(ft,'; ',s);
      with Feld do for i:=0 to 8 do begin
        for j:=0 to 8 do begin
          s:=Trim(Cells[j,i]);
          if j>0 then s:=','+s;
          if length(s)>0 then write(ft,s);
          end;
        writeln(ft);
        end;
      CloseFile(ft);
      btnReset.Enabled:=true;
      lblFile.Caption:=StripPath(TextName,55);
      lblFile.Hint:=Comment;
      DataLoaded:=true;
      end;
    end;
  end;

{-----------------------------------------------------------------------}
procedure TSuForm.ShowStatus;
begin
  if Stop then StatusBar.Panels[0].Text:=_(' Solving cancelled by user')
  else StatusBar.Panels[0].Text:=Format(_('  Solution found: %u rounds, %u tries, %u back steps'),[CCount,TCount,BCount]);
  end;

{-----------------------------------------------------------------------}
// Solve
procedure TSuForm.btnStartClick(Sender: TObject);
var
  DigitGrid   : TDigitGrid;
  RowData,
  ColData,
  BoxData    : array [1..9] of TData;
  DataGrid   : TDataGrid;  // grid with possible digits
  fd         : TData;
  Marker,
  FRow,FCol,
  i,j,k,n,sp : integer;
  Ende,
  Found      : boolean;
  s          : string;
  gs         : TGridStack;

  // get box number from row an column
  function GetBox (Row,Col :integer) : integer;
  begin
    Result:=(Row-1) div 3+3*((Col-1) div 3)+1;
    end;

  // get next possible digit from data set
  function GetDigit(var Field : TData) : TDigit;
  var
    d : TDigit;
  begin
    Result:=0;
    with Field do for d:=1 to 9 do if d in Values then begin
      Result:=d;
      Exclude(Values,d); dec(Count);
      Break;
      end;
    end;

  // add digit from data set
  procedure IncludeDigit (var Data : TData; Digit : TDigit);
  begin
    with Data do if not (Digit in Values) then begin
      inc(Count);
      Include(Values,Digit);
      end;
    end;

  // remove digit from data set
  procedure ExcludeDigit (var Data : TData; Digit : TDigit);
  begin
    with Data do if Digit in Values then begin
      dec(Count);
      Exclude(Values,Digit);
      end;
    end;

  // reset data set
  procedure Reset (var Data : TData);
  begin
    with Data do begin
      Count:=0; Values:=[];
      end;
    end;

  // assemble all digits in a 3x3 box belonging to a special row and column
  procedure CalculateBox (Row,Col : integer);
  var
    n,i,j,r,c : integer;
    d   : TDigit;
  begin
    n:=GetBox(Row,Col);
    r:=3*((Row-1) div 3)+1;
    c:=3*((Col-1) div 3)+1;
    Reset(BoxData[n]);
    for i:=r to r+2 do for j:=c to c+2 do begin
      d:=DigitGrid[i,j];
      if d<>0 then IncludeDigit(BoxData[n],d);
      end;
    end;

  // assemble digits for all 3x3 boxes
  procedure CalculateBoxes;
  var
    i,j : integer;
    d   : TDigit;
  begin
    for i:=1 to 9 do Reset(BoxData[i]);
    for i:=1 to 9 do begin
      for j:=1 to 9 do begin
        d:=DigitGrid[i,j];
        if d<>0 then IncludeDigit(BoxData[GetBox(i,j)],d);
        end;
      end;
    end;

  // assmble digits in a row
  procedure CalculateRow (Row : integer);
  var
    j : integer;
    d : TDigit;
  begin
    Reset(RowData[Row]);
    for j:=1 to 9 do begin
      d:=DigitGrid[Row,j];
      if d<>0 then IncludeDigit(RowData[Row],d);
      end;
    end;

  // assemble digits in all rows
  procedure CalculateRows;
  var
    i : integer;
  begin
    for i:=1 to 9 do CalculateRow (i);
    end;

  // assemble digits in a column
  procedure CalculateCol (Col : integer);
  var
    i : integer;
    d : TDigit;
  begin
    Reset(ColData[Col]);
    for i:=1 to 9 do begin
      d:=DigitGrid[i,Col];
      if d<>0 then IncludeDigit(ColData[Col],d);
      end;
    end;

  // assmble digits in all columns
  procedure CalculateCols;
  var
    j : integer;
  begin
    for j:=1 to 9 do CalculateCol (j);
    end;

  // check puzzle for errors
  function CheckGrid : TCheckGrid;
  var
    i,j : integer;
    d   : TDigit;
    err : boolean;
  begin
    err:=false;
    for i:=1 to 9 do begin
      Reset(BoxData[i]); Reset(ColData[i]); Reset(RowData[i]);
      end;
    for i:=1 to 9 do for j:=1 to 9 do begin
      d:=DigitGrid[i,j];
      if (d<>0) then begin
        if d in ColData[j].Values then err:=true
        else IncludeDigit(ColData[j],d);
        if d in RowData[i].Values then err:=true
        else IncludeDigit(RowData[i],d);
        if d in BoxData[GetBox(i,j)].Values then err:=true
        else IncludeDigit(BoxData[GetBox(i,j)],d);
        end;
      end;
    if err then Result:=cgError
    else begin
      err:=true; Result:=cgCont;
      for i:=1 to 9 do err:=err and (ColData[i].Count=9);
      if err then for i:=1 to 9 do err:=err and (RowData[i].Count=9);
      if err then for i:=1 to 9 do for j:=1 to 9 do err:=err and (BoxData[GetBox(i,j)].Count=9);
      if err then Result:=cgFound;
      end;
    end;

  // assemble possible digits of an empty field
  function CalcValues (Row,Col : integer) : TData;
  var
    d : TDigit;
  begin
    with Result do begin
      Count:=9; Values:=[1..9];
      end;
    for d:=1 to 9 do if d in RowData[Row].Values then ExcludeDigit(Result,d);
    for d:=1 to 9 do if d in ColData[Col].Values then ExcludeDigit(Result,d);
    for d:=1 to 9 do if d in BoxData[GetBox(Row,Col)].Values then ExcludeDigit(Result,d);
    end;

  // check if solution was found
  function SolutionFound : boolean;
  var
    i,j : integer;
  begin
    Result:=true;
    for i:=1 to 9 do for j:=1 to 9 do Result:=Result and (DataGrid[i,j].Count=0);
    Result:=Result or Stop;
    end;

  // show grid with digits on desktop
  procedure UpdateGrid (const Data : TDigitGrid);
  var
    i,j : integer;
    d : TDigit;
  begin
    for i:=1 to 9 do for j:=1 to 9 do begin
      d:=Data[i,j];
      if d>0 then Feld.Cells[i-1,j-1]:=IntToStr(d)
      else Feld.Cells[i-1,j-1]:='';
      end;
    end;

begin
  Run:=true;
  if Step then Stop:=false
  else begin
    Step:=cbStep.Checked;
    if Step then cbStep.Enabled:=false;
    SetEditMode(false);
    with Feld do begin
      for i:=1 to 9 do for j:=1 to 9 do begin
        s:=Trim(Cells[i-1,j-1]);
        if length(s)=0 then begin
          DigitGrid[i,j]:=0;
          Objects[i-1,j-1]:=pointer(0);
          end
        else begin
          DigitGrid[i,j]:=TDigit(StrToInt(s));
          Objects[i-1,j-1]:=pointer(3);
          end;
        end;
      end;
    sp:=1; CCount:=0; BCount:=0; TCount:=0; FRow:=1; FCol:=1; Marker:=6;
    if CheckGrid=cgError then begin
      StatusBar.Panels[0].Text:=_(' Puzzle is erroneous!');
      ErrorDialog(Caption,_('The puzzle has errors!'));
      Step:=false;
      SetEditMode(true);
      Exit;
      end;
    repeat
      Stop:=Step;
      // Prepare grid for step mode
      if Step then with Feld do for i:=0 to 8 do for j:=0 to 8 do
        Objects[i,j]:=pointer(integer(Objects[i,j]) and 3);
      // compute the used digits in the 3x3 boxes, rows and columns
      CalculateBoxes;
      CalculateRows;
      CalculateCols;
      Found:=false;
      // Compute possible digits for empty fields and fill all fields with
      // only one candidate
      for i:=1 to 9 do for j:=1 to 9 do begin
        DataGrid[i,j]:=CalcValues(i,j);
        if (DigitGrid[i,j]=0) and (DataGrid[i,j].Count=1) then begin
          DigitGrid[i,j]:=GetDigit(DataGrid[i,j]);
          FRow:=i; FCol:=j;
          CalculateBox(i,j);
          CalculateRow(i);
          CalculateCol(j);
          with Feld do Objects[i-1,j-1]:=Pointer(integer(Objects[i-1,j-1]) or Marker);
          Found:=true;
          end;
        end;
//      if Found then TCount:=0;
      Ende:=CheckGrid=cgFound;
      if not (Found or Ende) then begin  // try fields with more than one candidate
        inc(TCount);
        repeat
          Found:=false; k:=0; n:=10;
          // search for field with minimum choices
          for i:=1 to 9 do for j:=1 to 9 do if (DigitGrid[i,j]=0) then begin
            k:=DataGrid[i,j].Count;
            if (k<n) then begin
              FRow:=i; FCol:=j;
              n:=k; Found:=true;
              end;
            end;
  {       // search for first empty field
           for i:=1 to 9 do for j:=1 to 9 do
           if (not Found) and (DigitGrid[i,j]=0) then begin
            FRow:=i; FCol:=j;
            Found:=true;
            end;
          }
          // try first value
          if Found then begin
            k:=GetDigit(DataGrid[FRow,FCol]);
            fd:=DataGrid[FRow,FCol];
            end;
          Found:=k>0;
          // go up stack and try next value
          while not Found and (sp>1) do begin
            dec(sp); inc(BCount);
            with gs[sp] do begin
              FRow:=r; FCol:=c;
              DigitGrid:=DGrid;
              fd:=CField;
              k:=GetDigit(fd);
              end;
            Found:=k>0;
            end;
          if Found then begin
            if (DataGrid[FRow,FCol].Count=0) and (sp=1) then Marker:=6 else Marker:=4;
            with Feld do Objects[FRow-1,FCol-1]:=Pointer(integer(Objects[FRow-1,FCol-1]) or Marker);
            end;
          until Found or (sp=1);
        if not Found and (sp=1) then begin
          StatusBar.Panels[0].Text:=_(' No solution found!');
          ErrorDialog(Caption,_('Could not find any solution!'));
          Step:=false;
          SetEditMode(true);
          Exit;
          end;
        with gs[sp] do begin
          r:=FRow; c:=FCol;
          DGrid:=DigitGrid;
          CField:=fd;
          if sp<StackMax then inc(sp);
          end;
        DigitGrid[FRow,FCol]:=k;
        end;
      if ((CCount mod 10)=0) or Step then begin
        UpdateGrid(DigitGrid);
  //      sleep(1000);
        StatusBar.Panels[0].Text:=Format(_(' Solving puzzle - Round: %u'),[CCount+1]);
        Application.ProcessMessages;
        end;
      inc(CCount);
      while Step and Stop do begin
        Sleep(100);
        Application.ProcessMessages;
        end;
      until Ende or Stop;
    ShowStatus;
    Run:=false;
    UpdateGrid(DigitGrid);
    Application.ProcessMessages;
    Step:=false;
    SetEditMode(true);
    end;
  end;

procedure TSuForm.FeldKeyPress(Sender: TObject; var Key: Char);
begin
  if Key=#$D then Key:=#0;
  end;

procedure TSuForm.FeldKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key=VK_Return then with Feld do begin
    if Col<ColCount-1 then Col:=Col+1
    else begin
      Col:=0;
      if Row<RowCount-1 then Row:=Row+1
      else Row:=0;
      end;
    end;
  end;

procedure TSuForm.itmEnuClick(Sender: TObject);
begin
  ChangeLanguage('en');
  if not DataLoaded then lblFile.Caption:=_('New puzzle')
  else ShowStatus;
  end;

procedure TSuForm.itmGerClick(Sender: TObject);
begin
  ChangeLanguage('de');
  if not DataLoaded then lblFile.Caption:=_('New puzzle')
  else ShowStatus;
  end;

end.
